pkgname           = 'django-biblio'
__description__   = 'django app for storing bibliographical references'
__version__       = '0.1'
__author__        = 'Michal Grochmal'
__author_email__  = 'grochmal at member dot fsf dot org'
__copyright__     = 'Michal Grochmal (C) 2014'
__license__       = 'GNU General Public License, version 3 or later'
__url__           = 'https://bitbucket.org/grochmal/django-biblio'
__date__          = '2014-11-20'




from django.contrib import admin
from django.contrib.sites.shortcuts import get_current_site
from django import forms
from django import http
from djbiblio import models
from conf.settings import MEDIA_URL

import bibtexparser as bp
import urlparse     as up

def work_bibdict(obj):
    bibtags = obj.bibtags.all()
    d = { b.tag : b.value for b in bibtags }
    return d

def export_bibtex(modeladmin, request, queryset):
    db = bp.bibdatabase.BibDatabase()
    for obj in queryset:
        db.entries.append(work_bibdict(obj))
    response = http.HttpResponse(content_type='application/x-bibtex')
    response['Content-Disposition'] = 'attachment; filename=biblio.bib'
    response.write(bp.dumps(db))
    return response
export_bibtex.short_description = u'Export as BibTeX'

class ProjectAdmin(admin.ModelAdmin):
    fields  =      [ [ 'name' , 'slug' ] ]
    list_display =   [ 'name' , 'slug' ]
admin.site.register(models.Project, ProjectAdmin)

class BibtagForm(forms.ModelForm):
    tag   = forms.CharField( max_length=20
                           , widget=forms.TextInput(attrs={'size':20}) )
    value = forms.CharField( max_length=1000
                           , widget=forms.TextInput(attrs={'size':70}) )
    class Meta:
        model  = models.Bibtag
        fields = [ 'tag' , 'value' ]
        #exclude = [ 'works' ]

class BibtagAdmin(admin.TabularInline):
    model = models.Bibtag
    form  = BibtagForm
#admin.site.register(models.Bibtag, BibtagAdmin)

#class AuthorAdmin(admin.StackedInline):
#    model = models.Author.works.through
#admin.site.register(models.Author, AuthorAdmin)

tarea = forms.Textarea(attrs={'rows':12,'cols':70})

class WorkForm(forms.ModelForm):
    mnemonic = forms.CharField( max_length=120
                              , widget=forms.TextInput(attrs={'size':70}) )
    slug     = forms.SlugField(max_length=30)
    abstract = forms.CharField(max_length=2000, required=False, widget=tarea)
    bibtex   = forms.CharField(max_length=3000, required=False, widget=tarea)
    workurl  = forms.URLField( label="Work's URL"
                             , required=False
                             , widget=forms.TextInput(attrs={'size':70}) )
    upload   = forms.FileField(max_length=80, required=False)
    def clean_slug(self):
        if self.obj:  # this is a change, don't bother checking uniqueness
            return self.cleaned_data['slug']
        try:
            models.Work.objects.get(slug=self.cleaned_data['slug'])
        except models.Work.DoesNotExist:
            return self.cleaned_data['slug']
        dups = 'This slug already exist, you must choose a different one'
        raise forms.ValidationError(dups, code='duplicatedworkslug')
    def clean(self):
        if not 'slug' in self.cleaned_data:  # bail
            return self.cleaned_data
        if self.cleaned_data.get('upload', False):
            url = ( 'http://' + get_current_site(self.req).domain + MEDIA_URL
                  + models.upload_work( self.cleaned_data['slug']
                                      , self.cleaned_data['upload'].name ) )
            self.cleaned_data['workurl'] = url
        if not self.cleaned_data.get('workurl', False):
            raise forms.ValidationError( 'You must provide a URL or a file'
                                       , code='urlfile' )
        return self.cleaned_data
    def save(self, *args, **kwargs):
        work = super(WorkForm, self).save(*args, **kwargs)
        work.save()
        self.save_m2m()
        records = bp.loads(self.cleaned_data['bibtex'])
        if records.entries:
            en = records.entries[0]
            if not en.get('type', False): en['type'] = 'misc'
        else:
            en = {}
            en['type'] = 'misc'
        en['id'] = self.cleaned_data['slug']
        for r in en.keys():
            # TODO change if existing
            models.Bibtag.objects.create(tag=r, value=en[r], work=work)
        return work
    class Meta:
        model   = models.Work
        fields  = [ 'mnemonic' , 'slug'
                  , 'projects' , 'abstract'
                  , 'workurl'  , 'upload'
                  , 'bibtex'
                  ]
        #exclude = ['projects']

class LocalFileFilter(admin.SimpleListFilter):
    title          = 'paper location'
    parameter_name = 'local'
    def lookups(self, request, model_admin):
        return (
            ( 'local'  , 'local files'  ),
            ( 'remote' , 'remote files' ),
               )
    def queryset(self, request, queryset):
        if self.value() == 'local':
            return queryset.exclude(upload='')
        if self.value() == 'remote':
            return queryset.filter(upload='')
        return queryset

class WorkAdmin(admin.ModelAdmin):
    actions = [ export_bibtex ]
    inlines = [ BibtagAdmin ] # , AuthorAdmin ]
    form    = WorkForm
    readonly_fields = ['show_url']
    fields  = [ [ 'mnemonic' , 'slug'     ]
              , [ 'projects' , 'abstract' ]
              , [ 'workurl'  , 'show_url' ]
              , 'upload'
              , 'bibtex'
              ]
    list_filter   = [ LocalFileFilter ]
    list_display  = [ 'mnemonic' , 'slug', 'show_url' ]
    search_fields = [ 'mnemonic' , 'slug', 'abstract' ]
    def get_form(self, request, obj=None, **kwargs):
        form = super(WorkAdmin, self).get_form(request, obj, **kwargs)
        form.req = request
        form.obj = obj
        return form
    def show_url(self, inst):
        purl   = up.urlparse(inst.workurl)
        domain = '{uri.netloc}'.format(uri=purl)
        return '<a href="%s" target="_blank">%s</a>' % (inst.workurl , domain)
    show_url.allow_tags        = True
    show_url.short_description = u'View paper'
admin.site.register(models.Work, WorkAdmin)




from django.db import models
from django import forms
try:
    from conf.settings import DJBIBLIO_MAX_SIZE
except ImportError:
    DJBIBLIO_MAX_SIZE = (33554432 , '32MB')

class Project(models.Model):
    name = models.CharField(max_length=120)
    slug = models.SlugField(unique=True)
    def __unicode__(self):
        return self.name
#    def get_absolute_url(self):
#        return '/biblio/project/%d/' % self.id

class WorkFileField(models.FileField):
    def __init__(self, *args, **kwargs):
        self.max_size = kwargs.pop('max_size', 0)
        self.max_help = kwargs.pop('max_help', 'unlimited')
        super(WorkFileField, self).__init__(*args, **kwargs)
    def clean(self, *args, **kwargs):
        data = super(WorkFileField, self).clean(*args, **kwargs)
        if self.max_size and self.max_size < data.size:
            ftb = 'Sorry, file must be smaller than %s' % self.max_help
            raise forms.ValidationError(ftb, code='filetoobig')
        return data

def upload_work(slug, file):
    return 'work/' + slug + '/' + file
def upload_work_inst(inst, file):
    return upload_work(inst.slug, file)

class Work(models.Model):
    mnemonic = models.CharField(max_length=120)
    slug     = models.SlugField(unique=True)
    abstract = models.TextField(max_length=2000, blank=True)
    workurl  = models.URLField(verbose_name="Work's URL")
    upload   = WorkFileField( upload_to=upload_work_inst
                            , blank=True
                            , max_size=DJBIBLIO_MAX_SIZE[0]
                            , max_help=DJBIBLIO_MAX_SIZE[1] )
    projects = models.ManyToManyField(Project, related_name='works')
    def __unicode__(self):
        return self.mnemonic
#    def get_absolute_url(self):
#        return '/biblio/work/%d/' % self.id

class Bibtag(models.Model):
    tag   = models.CharField(verbose_name='Field Name', max_length=20)
    value = models.TextField(max_length=1000)
    work  = models.ForeignKey(Work, related_name='bibtags')
    def __unicode__(self):
        return '%s : %s' % ( self.tag , self.value[0:60] )
#    def get_absolute_url(self):
#        return '/biblio/bibtag/%d/' % self.id

# Author is *not* meant as a substitute for the author bibtex tag, neither will
# it be updated if the tag is updated.  It is only meant as and extra index
# that is generated when the input bibtex is parsed for the first time.
class Author(models.Model):
    name  = models.CharField(max_length=255, unique=True)
    works = models.ManyToManyField(Work, related_name='authors')
    def __unicode__(self):
        return self.name
#    def get_absolute_url(self):
#        return '/biblio/author/%d/' % self.id




#!/usr/bin/env python

try:
    from setuptools import setup
except ImportError:
    from distutils import setup
import os
import djbiblio

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

# Full list of classifiers can be found here:
# https://pypi.python.org/pypi?%3Aaction=list_classifiers
CLS = \
 [ 'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)'
 , 'Development Status :: 3 - Alpha'
 , 'Environment :: Web Environment'
 , 'Framework :: Django'
 , 'Intended Audience :: Science/Research'
 , 'Operating System :: OS Independent'
 , 'Programming Language :: Python'
 , 'Topic :: Text Processing :: Markup :: LaTeX'
 ]

REQS = [ 'django >= 1.6'
       , 'bibtexparser >= 0.6.0'
       ]

setup( name             = djbiblio.pkgname
     , description      = djbiblio.__description__
     , version          = djbiblio.__version__
     , author           = djbiblio.__author__
     , author_email     = djbiblio.__author_email__
     , license          = djbiblio.__license__
     , url              = djbiblio.__url__
     , long_description = read('README')
     , packages         = [ 'djbiblio' ]
     , classifiers      = CLS
     , install_requires = REQS
     )

